'use strict'

exports.__esModule = true
exports.default = exports.useDebounce = exports.useDebounceCallback = void 0

var _react = require('react')

var _requestTimeout = require('@essentials/request-timeout')

const useDebounceCallback = (callback, wait = 100, leading = false) => {
  const timeout = (0, _react.useRef)(null) // cleans up pending timeouts when the function changes

  function _ref() {
    if (timeout.current !== null) {
      ;(0, _requestTimeout.clearRequestTimeout)(timeout.current)
      timeout.current = null
    }
  }

  ;(0, _react.useEffect)(() => _ref, [callback, wait])
  return (0, _react.useCallback)(
    function() {
      // eslint-disable-next-line @typescript-eslint/no-this-alias
      const self = this // eslint-disable-next-line prefer-rest-params

      const args = arguments
      if (timeout.current === null && leading) callback.apply(self, args)
      else if (timeout.current !== null)
        (0, _requestTimeout.clearRequestTimeout)(timeout.current)
      timeout.current = (0, _requestTimeout.requestTimeout)(() => {
        timeout.current = null
        !leading && callback.apply(self, args)
      }, wait)
    },
    [callback, wait]
  )
}

exports.useDebounceCallback = useDebounceCallback

const useDebounce = (initialState, wait, leading) => {
  const [state, setState] = (0, _react.useState)(initialState)
  return [state, useDebounceCallback(setState, wait, leading)]
}

exports.useDebounce = useDebounce
var _default = useDebounce
exports.default = _default
